@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 1998, 2019 IBM Corp. and others
+ * Copyright (c) 1998, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -25,6 +25,8 @@ import java.io.*;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.IdentityHashMap;
+import java.util.Set;
 
 import com.ibm.oti.util.Msg;
 import com.ibm.oti.util.Util;
@@ -77,7 +79,7 @@ public class Throwable implements java.io.Serializable {
 	 * The list containing the exceptions suppressed 
 	 */
 	private List<Throwable> suppressedExceptions = Collections.EMPTY_LIST;
-	private transient boolean enableWritableStackTrace = true;
+	private transient boolean disableWritableStackTrace;
 	
 /**
  * Constructs a new instance of this class with its 
@@ -162,7 +164,7 @@ protected Throwable(String detailMessage, Throwable throwable,
 	}
 	
 	if (enableWritableStackTrace == false)	{
-		this.enableWritableStackTrace = false;
+		this.disableWritableStackTrace = true;
 	} else {
 		fillInStackTrace();
 	}
@@ -232,7 +234,7 @@ public void setStackTrace(StackTraceElement[] trace) {
 		}
 	}
 	
-	if (enableWritableStackTrace == false) {
+	if (disableWritableStackTrace) {
 		return;
 	}
 	
@@ -279,7 +281,7 @@ private static int countDuplicates(StackTraceElement[] currentStack, StackTraceE
  */
 StackTraceElement[] getInternalStackTrace() {
 
-	if (!enableWritableStackTrace) {
+	if (disableWritableStackTrace) {
 		return	ZeroStackTraceElementArray;
 	}
 
@@ -298,14 +300,7 @@ StackTraceElement[] getInternalStackTrace() {
  *				The stream to write the walkback on.
  */
 public void printStackTrace (PrintStream err) {
-    StackTraceElement[] stack;
-    stack = printStackTrace(err, null, 0, false);
-
-	Throwable throwable = getCause();
-	while (throwable != null && stack != null) {
-        stack = throwable.printStackTrace(err, stack, 0, false);
-		throwable = throwable.getCause();
-	}
+	printStackTraceHelper(err);
 }
 
 /**
@@ -316,14 +311,33 @@ public void printStackTrace (PrintStream err) {
  *				The writer to write the walkback on.
  */
 public void printStackTrace(PrintWriter err) {
-    StackTraceElement[] stack;
-    stack = printStackTrace(err, null, 0, false);
+	printStackTraceHelper(err);
+}
 
-    Throwable throwable = getCause();
-    while (throwable != null && stack != null) {
-        stack = throwable.printStackTrace(err, stack, 0, false);
-        throwable = throwable.getCause();
-    }
+/**
+ * Outputs representation of the receiver's
+ * walkback on the Appendable specified by the argument.
+ *
+ * @param	appendable Appendable 
+ *		The Appendable object to the walkback will be written.
+ */
+private void printStackTraceHelper(Appendable appendable) {
+	StackTraceElement[] stack;
+	Set<Throwable> exceptionChainSet = null;
+	try {
+		exceptionChainSet = Collections.newSetFromMap(new IdentityHashMap<Throwable, Boolean>());
+	} catch(OutOfMemoryError e) {
+		/* If OOM is thrown when creating exception set, then we won't be able to check for circular exception chain,
+		 * which can cause OOM to be thrown again. This should be ok as we are already running out of heap memory.
+		 */
+	}
+	stack = printStackTrace(appendable, null, 0, false, exceptionChainSet);
+
+	Throwable throwable = getCause();
+	while (throwable != null && stack != null) {
+		stack = throwable.printStackTrace(appendable, stack, 0, false, exceptionChainSet);
+		throwable = throwable.getCause();
+	}
 }
 
 /**
@@ -408,7 +422,7 @@ private void readObject(ObjectInputStream s)
 	throws IOException, ClassNotFoundException	{
 	s.defaultReadObject();
 	
-	enableWritableStackTrace = (stackTrace != null);
+	disableWritableStackTrace = (stackTrace == null);
 	
 	if (stackTrace != null) {
 		if (stackTrace.length == 1) {
@@ -431,18 +445,31 @@ private void readObject(ObjectInputStream s)
 	
 	
 	if (suppressedExceptions != null) {
-		if (suppressedExceptions.size() == 0) {
-			suppressedExceptions = Collections.EMPTY_LIST;
-		} else {
-			for (Throwable t : suppressedExceptions) {
-				if (t == null) {
-					// K0561 = Null entries not permitted in suppressedExceptions serial stream
-					throw new NullPointerException(com.ibm.oti.util.Msg.getString("K0561")); //$NON-NLS-1$
-				} else if (t == this) {
-					// K0562 = Self-pointers not permitted in suppressedExceptions serial stream
-					throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K0562")); //$NON-NLS-1$
+		List<Throwable> newList = Collections.EMPTY_LIST;
+		try {
+			Module classModule = suppressedExceptions.getClass().getModule();
+			if (Object.class.getModule().equals(classModule)) {
+				int listSize = suppressedExceptions.size();
+				if (listSize != 0) {
+					newList = new ArrayList<Throwable>(listSize);
+					for (Throwable t : suppressedExceptions) {
+						if (t == null) {
+							// K0561 = Null entries not permitted in suppressedExceptions serial stream
+							throw new NullPointerException(com.ibm.oti.util.Msg.getString("K0561")); //$NON-NLS-1$
+						} else if (t == this) {
+							// K0562 = Self-pointers not permitted in suppressedExceptions serial stream
+							throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K0562")); //$NON-NLS-1$
+						} else {
+							newList.add(t);
+						}
+					}
 				}
+			} else {
+				// K0C00 = Non-standard List class not permitted in suppressedExceptions serial stream
+				throw new java.io.StreamCorruptedException(com.ibm.oti.util.Msg.getString("K0C00")); //$NON-NLS-1$
 			}
+		} finally {
+			suppressedExceptions = newList;
 		}
 	}
 }
@@ -463,46 +490,77 @@ private void readObject(ObjectInputStream s)
  * 			number of indents (\t) to be printed
  * @param	suppressed	
  * 			if this is an exception suppressed
+ * @param	exceptionChainSet
+ * 			set of exceptions in the exception chain
  * 
  * @return	an array of stack trace elements printed 
  * 
  */
 private StackTraceElement[] printStackTrace(
-		Appendable err, StackTraceElement[] parentStack, int indents, boolean suppressed) {
+		Appendable err, StackTraceElement[] parentStack, int indents, boolean suppressed, Set<Throwable> exceptionChainSet) {
 	if (err == null) throw new NullPointerException();
-    StackTraceElement[] stack;
-    boolean outOfMemory = this instanceof OutOfMemoryError;
-    if (parentStack != null) {
-	    if (suppressed) {
-	    	appendTo(err, "Suppressed: ", indents); //$NON-NLS-1$
-	    } else {
-	    	appendTo(err, "Caused by: ", indents); //$NON-NLS-1$
-	    }
+	StackTraceElement[] stack;
+	boolean outOfMemory = this instanceof OutOfMemoryError;
+	if ((exceptionChainSet != null) && (exceptionChainSet.contains(this))) {
+		if (!outOfMemory) {
+			try {
+				appendTo(err, "\t[CIRCULAR REFERENCE:" + toString() + "]", 0); //$NON-NLS-1$
+			} catch(OutOfMemoryError e) {
+				outOfMemory = true;
+			}
+		}
+		if (outOfMemory) {
+			appendTo(err, "\t[CIRCULAR REFERENCE:"); //$NON-NLS-1$
+			try {
+				appendTo(err, getClass().getName());
+			} catch(OutOfMemoryError e) {
+				appendTo(err, "java.lang.OutOfMemoryError(?)");
+			}
+			appendTo(err, "]");
+		}
+		appendLnTo(err);
+		return null;
 	}
-    if (!outOfMemory) try {
-    	appendTo(err, toString());
-    } catch(OutOfMemoryError e) {
-        outOfMemory = true;
-    }
-    if (outOfMemory) {
-        try {
-            appendTo(err, getClass().getName());		
-        } catch(OutOfMemoryError e) {
+	try {
+		exceptionChainSet.add(this);
+	} catch(OutOfMemoryError e) {
+		/* If OOM is thrown when adding Throwable to exception set, then we may not be able to identify circular exception chain,
+		 * which can cause OOM to be thrown again. This should be ok as we are already running out of heap memory.
+		 */
+	}
+	if (parentStack != null) {
+		if (suppressed) {
+			appendTo(err, "Suppressed: ", indents); //$NON-NLS-1$
+		} else {
+			appendTo(err, "Caused by: ", indents); //$NON-NLS-1$
+		}
+	}
+	if (!outOfMemory) {
+		try {
+			appendTo(err, toString());
+		} catch(OutOfMemoryError e) {
+			outOfMemory = true;
+		}
+	}
+	if (outOfMemory) {
+		try {
+			appendTo(err, getClass().getName());
+		} catch(OutOfMemoryError e) {
 			outOfMemory = true;
 			appendTo(err, "java.lang.OutOfMemoryError(?)");			 //$NON-NLS-1$
-        }
-        try {
+		}
+		try {
 			String message = getLocalizedMessage();
 			if (message != null) {
 				appendTo(err, ": "); //$NON-NLS-1$
 				appendTo(err, message);
 			}
-        } catch(OutOfMemoryError e) {
+		} catch(OutOfMemoryError e) {
 			outOfMemory = true;
-        }
-    }
+		}
+	}
 	appendLnTo(err);
-    int duplicates = 0;
+	int duplicates = 0;
 	try {
 		// Don't use getStackTrace() as it calls clone()
 		// Get stackTrace, in case stackTrace is reassigned
@@ -510,34 +568,38 @@ private StackTraceElement[] printStackTrace(
 		if (parentStack != null) {
 			duplicates = countDuplicates(stack, parentStack);
 		}
-	} catch (OutOfMemoryError e) {
+	} catch(OutOfMemoryError e) {
 		appendTo(err, "\tat ?", indents); //$NON-NLS-1$
 		appendLnTo(err);
 		return null;
 	}
-    for (int i=0; i < stack.length - duplicates; i++) {
-		if (!outOfMemory) try {
-            appendTo(err, "\tat " + stack[i], indents);			 //$NON-NLS-1$
-        } catch(OutOfMemoryError e) {
-			outOfMemory = true;
-        }
+	for (int i=0; i < stack.length - duplicates; i++) {
+		if (!outOfMemory) {
+			try {
+				appendTo(err, "\tat " + stack[i], indents);			 //$NON-NLS-1$
+			} catch(OutOfMemoryError e) {
+				outOfMemory = true;
+			}
+		}
 		if (outOfMemory) {
 			appendTo(err, "\tat ", indents); //$NON-NLS-1$
 			Util.printStackTraceElement(stack[i], null, err, false);
-        }
+		}
 		appendLnTo(err);		
-    }
-    if (duplicates > 0) {
-		if (!outOfMemory) try {
-			appendTo(err, "\t... " + duplicates + " more", indents); //$NON-NLS-1$ //$NON-NLS-2$
-		} catch(OutOfMemoryError e) {
-			outOfMemory = true;
+	}
+	if (duplicates > 0) {
+		if (!outOfMemory) {
+			try {
+				appendTo(err, "\t... " + duplicates + " more", indents); //$NON-NLS-1$ //$NON-NLS-2$
+			} catch(OutOfMemoryError e) {
+				outOfMemory = true;
+			}
 		}
 		if (outOfMemory) {
 			appendTo(err, "\t... ", indents); //$NON-NLS-1$
 			appendTo(err, duplicates);
 			appendTo(err, " more"); //$NON-NLS-1$
-        }		
+        	}		
 		appendLnTo(err);
 	}
     
@@ -545,11 +607,11 @@ private StackTraceElement[] printStackTrace(
 		if (suppressedExceptions != null) {
 			for (Throwable t : suppressedExceptions) {
 				StackTraceElement[] stackSuppressed;
-				stackSuppressed = t.printStackTrace(err, stack, indents + 1, true);
+				stackSuppressed = t.printStackTrace(err, stack, indents + 1, true, exceptionChainSet);
 				
 				Throwable throwableSuppressed = t.getCause();
 				while (throwableSuppressed != null && stackSuppressed != null) {
-					stackSuppressed = throwableSuppressed.printStackTrace(err, stackSuppressed, indents + 1, false);
+					stackSuppressed = throwableSuppressed.printStackTrace(err, stackSuppressed, indents + 1, false, exceptionChainSet);
 					throwableSuppressed = throwableSuppressed.getCause();
 				}
 			}
