@@ -1097,6 +1097,11 @@ public abstract class MethodHandle
 					throw WrongMethodTypeException.newWrongMethodTypeException(type, callsiteType);
 				}
 				result = cs.dynamicInvoker();
+			} 
+			else {
+				/* The result of the resolution of a dynamically-computed call site must not be null. */
+				// K0A02 = Bootstrap method returned null.
+				throw new ClassCastException(Msg.getString("K0A02")); //$NON-NLS-1$
 			}
 		} catch(Throwable e) {
 
@@ -1363,7 +1368,8 @@ public abstract class MethodHandle
 	}
 	
 	void customize() {
-		throw OpenJDKCompileStub.OpenJDKCompileStubThrowError();
+		// this is an empty implementation to satisfy RI specific method calls
+		// https://github.com/eclipse/openj9/issues/7080
 	}
 	
 	void updateForm(LambdaForm lf) {
