@@ -178,6 +178,7 @@ public final class ModuleBootstrap {
 
         boolean haveModulePath = (appModulePath != null || upgradeModulePath != null);
         boolean needResolution = true;
+        boolean canArchive = false;
 
         // If the java heap was archived at CDS dump time and the environment
         // at dump time matches the current environment then use the archived
@@ -192,7 +193,6 @@ public final class ModuleBootstrap {
             systemModuleFinder = archivedModuleGraph.finder();
             needResolution = (traceOutput != null);
         } else {
-            boolean canArchive = false;
             if (!haveModulePath && addModules.isEmpty() && limitModules.isEmpty()) {
                 systemModules = SystemModuleFinders.systemModules(mainModule);
                 if (systemModules != null && !isPatched) {
@@ -212,12 +212,6 @@ public final class ModuleBootstrap {
                 systemModules = new ExplodedSystemModules();
                 systemModuleFinder = SystemModuleFinders.ofSystem();
             }
-
-            // Module graph can be archived at CDS dump time. Only allow the
-            // unnamed module case for now.
-            if (canArchive && (mainModule == null)) {
-                ArchivedModuleGraph.archive(mainModule, systemModules, systemModuleFinder);
-            }
         }
 
         Counters.add("jdk.module.boot.1.systemModulesTime", t1);
@@ -359,8 +353,12 @@ public final class ModuleBootstrap {
         if (needResolution) {
             cf = JLMA.resolveAndBind(finder, roots, traceOutput);
         } else {
-            Map<String, Set<String>> map = systemModules.moduleReads();
-            cf = JLMA.newConfiguration(systemModuleFinder, map);
+            if (archivedModuleGraph != null) {
+                cf = archivedModuleGraph.configuration();
+            } else {
+                Map<String, Set<String>> map = systemModules.moduleReads();
+                cf = JLMA.newConfiguration(systemModuleFinder, map);
+            }
         }
 
         // check that modules specified to --patch-module are resolved
@@ -373,6 +371,7 @@ public final class ModuleBootstrap {
 
         Counters.add("jdk.module.boot.4.resolveTime", t4);
 
+
         // Step 5: Map the modules in the configuration to class loaders.
         // The static configuration provides the mapping of standard and JDK
         // modules to the boot and platform loaders. All other modules (JDK
@@ -446,6 +445,13 @@ public final class ModuleBootstrap {
         ((BuiltinClassLoader)platformLoader).initializeSharedClassesSupport();                  //OpenJ9-shared_classes_misc
         ((BuiltinClassLoader)appLoader).initializeSharedClassesSupport();                               //OpenJ9-shared_classes_misc
 
+        // Module graph can be archived at CDS dump time. Only allow the
+        // unnamed module case for now.
+        if (canArchive && (mainModule == null)) {
+            ArchivedModuleGraph.archive(mainModule, systemModules,
+                                        systemModuleFinder, cf);
+        }
+
         // total time to initialize
         Counters.add("jdk.module.boot.totalTime", t0);
         Counters.publish();
@@ -578,7 +584,7 @@ public final class ModuleBootstrap {
         // the system property is removed after decoding
         String value = getAndRemoveProperty(prefix + index);
         if (value == null) {
-            return Collections.emptySet();
+            return Set.of();
         } else {
             Set<String> modules = new HashSet<>();
             while (value != null) {
@@ -599,7 +605,7 @@ public final class ModuleBootstrap {
     private static Set<String> limitModules() {
         String value = getAndRemoveProperty("jdk.module.limitmods");
         if (value == null) {
-            return Collections.emptySet();
+            return Set.of();
         } else {
             Set<String> names = new HashSet<>();
             for (String name : value.split(",")) {
@@ -851,7 +857,7 @@ public final class ModuleBootstrap {
         // the system property is removed after decoding
         String value = getAndRemoveProperty(prefix + index);
         if (value == null)
-            return Collections.emptyMap();
+            return Map.of();
 
         Map<String, List<String>> map = new HashMap<>();
 
