@@ -75,10 +75,6 @@
  * @since 9
  */
 module java.base {
-    // source file: file:///Users/jenkins/workspace/build-scripts/jobs/jdk11u/jdk11u-mac-x64-openj9-macosXL/workspace/build/src/src/java.base/share/classes/module-info.java
-    //              file:///Users/jenkins/workspace/build-scripts/jobs/jdk11u/jdk11u-mac-x64-openj9-macosXL/workspace/build/src/src/java.base/macosx/classes/module-info.java.extra
-    //              file:///Users/jenkins/workspace/build-scripts/jobs/jdk11u/jdk11u-mac-x64-openj9-macosXL/workspace/build/src/src/java.base/unix/classes/module-info.java.extra
-    //              file:///Users/jenkins/workspace/build-scripts/jobs/jdk11u/jdk11u-mac-x64-openj9-macosXL/workspace/build/src/build/macosx-x86_64-normal-server-release/support/j9jcl_sources/java.base/share/classes/module-info.java.extra
     exports java.io;
     exports java.lang;
     exports java.lang.annotation;
@@ -158,7 +154,8 @@ module java.base {
     exports jdk.internal.loader to
         java.desktop,
         java.instrument,
-        java.logging;
+        java.logging,
+        java.naming;
     exports jdk.internal.logger to java.logging;
     exports jdk.internal.misc to
         java.desktop,
